/// <reference path="../untyped.d.ts" />
import { FilterActionAsync, Format, PandocJson } from "pandoc-filter";
/**
 * config options loaded from markdown frontmatter
 *
 * can also be specified as a command line option, e.g. `pandoc -V url2cite=all-links`
 **/
export declare type Configuration = {
    /**
     * if all-links then convert all links to citations. otherwise only parse pandoc citation syntax
     */
    url2cite?: "all-links" | "citation-only";
    /**
     * only relevant for links converted to citations (if urlcite=all-links)
     *
     * - cite-only: [text](href) becomes [@href]
     * - sup: [text](href) becomes [text](href)^[@href]
     * - normal: [text](href) becomes [text [@href]](href)
     *
     *  default: sup if html else normal
     */
    "url2cite-link-output"?: "cite-only" | "sup" | "normal";
    /**
     * location of the cache file
     * default: ./citation-cache.json (relative to invocation directory of pandoc)
     */
    "url2cite-cache"?: string;
    /**
     * Whether to allow citations without an accompanying url. Useful for manual citations that aren't
     * automatically found by url2cite and are managed manually or with a different tool
     */
    "url2cite-allow-dangling-citations"?: boolean;
    "url2cite-output-bib"?: string;
    /** escape characters in the cite keys that are not valid for biber */
    "url2cite-escape-ids"?: boolean;
};
/** type of the citation-cache.json file */
declare type Cache = {
    _info: string;
    urls: {
        [url: string]: {
            fetched: string;
            bibtex: string[];
            csl: CSL;
        };
    };
};
declare type CSL = {
    id: string;
};
export declare class Url2Cite {
    /** written to CWD from which pandoc is called */
    citationCachePath: string;
    cache: Cache;
    citekeys: {
        [key: string]: string;
    };
    getCslForUrlCached(url: string, escapeId: boolean): Promise<void>;
    extractCiteKeys: FilterActionAsync;
    /**
     * transform the pandoc document AST
     * - replaces links with citations if `all-links` is active or they are marked with `url2cite` class/title
     * - replaces citekeys with urls, fetches missing citations and writes them to cache
     */
    astTransformer: FilterActionAsync;
    transform(data: PandocJson, format: Format): Promise<PandocJson>;
    writeCache(): Promise<void>;
    writeBib(fname: string): Promise<void>;
}
export {};
//# sourceMappingURL=index.d.ts.map